from ScopusWp.extend import ExtensionInstallationController

import ScopusWp.scopus.persistency as ps
from ScopusWp.register import OriginType
from ScopusWp.scopus.config import PATH, NAME

import os


if __name__ == '__main__':
    install = ExtensionInstallationController(NAME)
    install.create_mysql_database('__scopus', ps.BASE)

    install.explorer.session.add(OriginType(typeID=1, name='scopus'))

    template = install.load_template(os.path.join(PATH, 'templates', 'config.ini'))
    content = template.render(
        shorthand='SHORTHAND',
        ids=[],
        first_name='',
        last_name='',
        keywords=[],
        whitelist=[],
        blacklist=[]
    )

    install.create_file('config.ini', content)



import configparser
import logging
import os

import datetime
import pathlib
import threading

# Getting the absolute path to the folder of this very script file, so that can be used to open the config file, which
# is also located in this folder
PATH = os.path.dirname(os.path.realpath(__file__))
VERSION = '0.3.0'
PROJECT_PATH = '/home/jonas/IPE/ScopusWp' #'/home/jonas/PycharmProjects/ScopusWp/ScopusWp'

LOGGING_PATH = os.path.join(PROJECT_PATH, 'logs')
TEMPLATE_PATH = os.path.join(PATH, 'templates')

SCOPUS_LOGGING_EXTENSION = 'SCOPUS'
SQL_LOGGING_EXTENSION = 'SQL'


def _format_log_file_name(datetime_object, extension_string):

    # Formatting the string of the date
    date_string = _format_date(datetime_object)
    # Joining the date string with the string extension
    file_name_string = '-'.join([date_string, 'log'])
    file_name_string += '.txt'

    return file_name_string


def _format_date(datetime_object):
    date_string = '{}-{}-{}'.format(datetime_object.year, datetime_object.month, datetime_object.day)
    return date_string


def get_log_file_path(datetime_object, extension_string):
    file_name = _format_log_file_name(datetime_object, extension_string)
    file_path = os.path.join(LOGGING_PATH, file_name)
    return file_path


def log_exists(datetime_object, extension_string):
    file_path = get_log_file_path(datetime_object, extension_string)

    return os.path.exists(file_path) and os.path.isfile(file_path)


def init_logging():
    datetime_object = datetime.datetime.now()
    file_path = get_log_file_path(datetime_object, '')
    # Checking if the log already exists, if not
    if log_exists(datetime_object, ''):
        file_mode = 'a'
        with open(file_path, 'a') as file:
            file.write('\n\nSTARTING A NEW SESSION:\n\n')
    else:
        file_mode = 'w+'

    logging.basicConfig(level=logging.INFO,
                        filename=file_path,
                        filemode=file_mode,
                        format='%(asctime)s %(name)-40s %(levelname)-8s %(message)s')


class LoggingController:

    DATE_FORMAT = '%Y-%m-%d'
    DATETIME_FORMAT = '%Y-%m-%d %H:%M:%S'

    def __init__(self):
        self.config = Config.get_instance()

        self.log_file_path = pathlib.Path(LOGGING_PATH) / self.create_log_file_name()

    def init(self):
        self.prepare_log_file(self.log_file_path)

    def close(self):
        self.post_process_log_file(self.log_file_path)

    def prepare_log_file(self, path):

        # Checking if the path exists and if it does, that means there was a log written on that day already, then
        # opening the file in append mode
        # Getting the current datetime string
        datetime_object = datetime.datetime.now()
        datetime_string = datetime_object.strftime(self.DATETIME_FORMAT)

        line_string = '\n\nSTARTING A NEW SESSION @ "{}"\n\n'.format(datetime_string)

        file_mode = 'w+'
        with path.open(mode=file_mode) as file:
            file.write(line_string)

        file_path = str(path)
        logging.basicConfig(
            level=logging.DEBUG,
            filename=file_path,
            filemode='a',
            format='%(asctime)s %(name)-40s %(levelname)-8s %(message)s'
        )

    def post_process_log_file(self, path):
        datetime_object = datetime.datetime.now()
        datetime_string = datetime_object.strftime(self.DATETIME_FORMAT)

        line_string = '\n\nCLOSING SESSION @ "{}"\n\n'.format(datetime_string)

        with path.open(mode='a') as file:
            file.write(line_string)

    def create_activity_file_name(self):
        # Creating the date string from the current datetime object
        datetime_object = datetime.datetime.now()
        date_string = datetime_object.strftime(self.DATE_FORMAT)
        # The base name extension for the file name is saved as a entry in the config file
        base_name_string = self.config['LOGGING']['activity_log_name']

        # Joining the date string and the base string into the log file name
        name_string = '{}-{}.txt'.format(
            date_string,
            base_name_string
        )

        return name_string

    def create_log_file_name(self):
        # Creating the date string from the current datetime object
        datetime_object = datetime.datetime.now()
        date_string = datetime_object.strftime(self.DATE_FORMAT)
        # The base name extension for the file name is saved as a entry in the config file
        base_name_string = self.config['LOGGING']['debug_log_name']

        # Joining the date string and the base string into the log file name
        name_string = '{}-{}.txt'.format(
            date_string,
            base_name_string
        )

        return name_string


class LoggingObserver(threading.Thread):

    def __init__(self, logger_name):

        self.logger = logging.getLogger(logger_name)


class Config:
    """
    A singleton class for the access to the config object
    """
    _instance = None

    def __init__(self):
        pass

    @staticmethod
    def get_instance():

        if Config._instance is None:
            Config._instance = Config._create_instance()

        return Config._instance

    @staticmethod
    def _create_instance():
        # Getting the path to the config file
        path = Config._config_path()
        # Creating the config parser object and returning it
        config = configparser.ConfigParser()
        config.read(path)

        return config

    @staticmethod
    def _config_path():
        # Joining the config path
        path = os.path.join(PROJECT_PATH, 'config.ini')
        return path







from ScopusWp.config import Config, PATH, PROJECT_PATH
import pathlib
import json


ID_JSON_TEMPLATE = (
    '{\n'
    '"counter": 1,\n'
    '"used": [],\n'
    '"unused": []\n'
    '}'
)


class IDManagerInterface:

    def new(self):
        raise NotImplementedError()

    def delete(self, publication_id):
        raise NotImplementedError

    @property
    def all(self):
        raise NotImplementedError()

    @property
    def used(self):
        raise NotImplementedError()

    @property
    def unused(self):
        raise NotImplementedError()


class IDManager(IDManagerInterface):

    def __init__(self, name):
        self.name = name
        self.path = pathlib.Path(PROJECT_PATH) / 'id_{}.json'.format(self.name)

        self._dict = {}
        self.load()

        self.used_ids = self._dict['used']
        self.unused_ids = self._dict['unused']
        self.pointer = self._dict['counter']

    def exists(self):
        return self.path.exists() and self.path.is_file()

    def save(self):
        """
        Saves the the content dict, which contains the info about the used ids into a json file in the project folder

        :return: void
        """
        with self.path.open(mode='w') as file:
            content_dict = {
                'used': self.used_ids,
                'unused': self.unused_ids,
                'counter': self.pointer
            }
            json.dump(content_dict, file)

    def new(self):
        """
        Creates and returns a new id to use.

        Updates the created id into the list of used ids directly and increments the pointer, that indicates the amount
        of used ids.
        :return: The int id that has been created
        """
        self.pointer += 1
        self.used_ids.append(self.pointer)
        return self.pointer

    def delete(self, publication_id):
        if publication_id in self.used_ids:
            self.used_ids.remove(publication_id)
            self.unused_ids.append(publication_id)

    def load(self):
        """
        Loads the information about the used ids from the json file in the project folder.

        :return: the dict, that contains the list if used and unused ids as well as the pointer for the amount of ids
        """
        # First checking if the file already exists. If not creates a new one from the template and loads that
        if not self.exists():
            with self.path.open(mode='w+') as file:
                file.write(ID_JSON_TEMPLATE)
                file.flush()

        # Loading the content of the file as a single dict
        with self.path.open(mode='r') as file:
            self._dict = json.load(file)

    @property
    def all(self):
        return self.used_ids

    @property
    def used(self):
        return self.used_ids

    @property
    def unused(self):
        return self.unused_ids